/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef EVALUATION_H
#define EVALUATION_H

/** evaluation.h
 * 
 * Ce fichier contient les fonction permettant d'évaluer la qualité des particules générée par le traqueur.
 * 
 * On travaille ici en 2D!
 * 
 * \author BURGER Brice
 * \since 01/2007
 */


#include "gestParams.h"
#include "mesureCouleur.h"



class evaluation
{

	public:
	
	/** Constructeur */
	evaluation();
	
	/** Destructeur */
	~evaluation();
	
	/** Cette fonction initialise les fonctions d'évaluation et les tableaux et classes utilisés
	 * 
	 */
	void init(IplImage* img, CvPoint centre, int rayon);
	
	/** calcule la vraisemblance d'une ellipse
	 * 
	 * on commence par discrétiser l'ellipse en nbPtEchantillonnageEllipse points de contour,
	 * puis on calcule l'ensemble des vraissemblances qu'on veut prendre en compte
	 * 
	 * \param imgprobapeau : l'image de probabilité de peau (en niveaux de gris donc)
	 * \param imgmvt : l'image de mouvement
	 * \param imgdist : l'image de distance
	 * \param centre,axes et angle : ellipse à évaluer
	 * \param les unités sont le pixel et le degré
	 * 
	 * \return retourne la vraissemblance calculée
	 * 
	 * Si une image = NULL, ce type de vraissemblance ne sera pas prise en compte
	 */
	double evalEllipse(IplImage* imgprobapeau, IplImage* imgmvt, IplImage* imgdist, CvPoint centre, CvSize axes, double angle);
	
	/** calcule la vraisemblance d'un cercle
	 * 
	 * on commence par discrétiser le cercle en nbPtEchantillonnageCercle points de contour,
	 * puis on calcule l'ensemble des vraissemblances qu'on veut prendre en compte
	 * 
	 * \param imgprobapeau : l'image de probabilité de peau (en niveaux de gris donc)
	 * \param imgcoul : image en cours
	 * \param imgdist : l'image de distance
	 * \param centre,rayon : cercle à évaluer
	 * \param les unités sont le pixel et le degré
	 * 
	 * \return retourne la vraissemblance calculée
	 * 
	 * Si imgcoul ou imgprobapeau = NULL, ce type de vraissemblance ne sera pas prise en compte
	 */
	double evalCercle(IplImage* imgprobapeau, IplImage* imgcoul, IplImage* imgdist, CvPoint centre, int rayon);
	
	/** met à jour ce qui peut l'être : (pour l'instant que l'histogramme de couleur)
	 * 
	 */
	void MAJeval();
	
	/** fonction calculant le poids à donner à une particule en fonction de sa vraissemblance et de celle des autres particules :
	 * 
	 * \param V : vraissemblance de la particule
	 * \param Vmax : vraissemblance maximale
	 */
	double poids(double V, double Vmax);
	
	
	/**********************/
	/* fonctions internes */
	/**********************/
	
	private:
	
	/** calcule la vraisemblance d'une forme discrétisée selon une méthode inspirée de THAYANANTHAN
	 * 
	 * on calcule la moyenne des probabilités de peau vers l'intérieur et vers l'extérieur.
	 * au final on cherche à avoir une proba maximale à l'intérieur de l'ellipse et minimale à l'extérieur
	 * 
	 * \param img : l'image de probabilité de peau (en niveaux de gris donc)
	 * \param N : nombre de points discrétisés
	 * \param centre : centre de la forme
	 * \param les unités sont le pixel et le degré
	 * 
	 * \return retourne g = exp( 1 - 1/moyenne des probas peau intérieures ) * exp( moyenne des probas peau extérieures / (moyenne des probas peau extérieures-1) )
	 */
	double proximitepeau(IplImage* img, int N, CvPoint centre);
	
	/** calcule le "taux de mouvement" dans une forme discrétisée
	 * 
	 * on calcule la moyenne de la valeur des pixels (sur les orthogonales à la forme aux points discrétisés) sur l'image de mouvement.
	 * cette moyenne est considérée comme étant le "taux de mouvement"
	 * 
	 * \param img : l'image de mouvement
	 * \param N : nombre de points discrétisés
	 * \param centre : centre de la forme
	 * \param les unités sont le pixel et le degré
	 * 
	 * \return retourne g = 1 - exp( - ( moyenne mouvement intérieur / (1 - moyenne mouvement intérieur) ) / sigmamvt )
	 */
	double tauxMouvement(IplImage* img, int N, CvPoint centre);
	
	/** calcule la distance aux contours d'une forme discrétisée
	 * 
	 * on calcule la moyenne de la valeur des pixels discrétisés sur l'image de distance.
	 * 
	 * \param img : l'image de distance
	 * \param N : nombre de points discrétisés
	 * 
	 * \return retourne g = exp( - moyenne distance / sigmacontour );
	 */
	double distanceContour(IplImage* img, int N, CvPoint centre);
	
	/** Discretisation de l'ellipse : 
	 * on la decoupe suivant des plans horizontaux pour en extraire des points de contours
	 *
	 * \param centre, axes et angle : caracteristiques de l'ellipse
	 * \param nb_pt : nombre de points a obtenir
	 * \return retourne le nombre de points effectivement crees
	 */
	int discretise_ellipse(CvPoint centre, CvSize axes, double angle, int nb_pt);
	
	/** Discrétisation d'un cercle : 
	 *
	 * \param centre, rayon : caractéristiques du cercle
	 * \param nb_pt : nombre de points a obtenir
	 * \return retourne le nombre de points effectivement créés
	 */
	int discretise_cercle(CvPoint centre, int rayon, int nb_pt);
	
	/* variables générales */
	CvPoint* pts;//coordonnées des pts échantillonnés
	MesureCouleur* mesurecoul;	//... c'est pas génial comme çà, mais on a besoin d'une mesure par image, on partira donc du principe 
	MesureCouleur* mesurecoul1;	//... qu'à chaque calcul de mesure de couleur on change d'image (et on alterne donc 1/2)
	MesureCouleur* mesurecoul2;	//... quand evaluation sera une classe çà ne posera plus de problème (on en fera alors un pour l'image gauche et un pour la droite)
	int aaaaaaaaa;
	MesureCouleur* mesurecoulREF;
	
	/* autres variables (variables internes des fonctions) */
	int nbpt;
	double eval,Poids;
	double I,theta,pas,a,b,c,s,Y,A,A4,B,C,D,inv2A,pDelta,delta;
	double dist,g;
	int N,J,K;
	int j;
	int deltaX, deltaY;
	unsigned int dX, dY;
	int Mint,Pint,Pext;
	double moyenneint;
	double moyenneext;
};

#endif
