/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* particules.cpp
 * 
 * Cette classe contient les fonctions directement relatives aux particules : génération, sélection, etc.
 * Elle gère également les particules courantes et passées du filtre particulaire.
 * 
 * Les particules sont de dimension nbvar.
 * Certaines fonctions ne sont pas génériques, il faudra donc créer un nouveau cas dans celles-ci pour chaque dimension non encore prise en compte
 * Ces fonction sont : evolutionDapresDetection() et evolutionDapresDynamique(), aini que le constructeur (variances)
 * 
 * Les unités sont maintenant le mm et le degré. Ils sont toujours exprimés en int.
 * 
 * Auteur : BURGER Brice
 * Date : 01/2007
 */


#include "particules.h"




/* constructeur :
 * 
 * Nbvar : nombre de variables des particules à créer
 */
Particules::Particules(int Nbvar) {
	
	nbvar = Nbvar;
	
	register int i;
	/* allocation de mémoire pour les particules et leur poids */
	particules = new int*[nbpart];
	particulesPrec = new int*[nbpart];
	particuleDetectees = new int*[nbpart];
	for(i=0;i<nbpart;i++) {
		particules[i] = new int[nbvar];
		particulesPrec[i] = new int[nbvar];
		particuleDetectees[i] = new int[nbvar];
	}
	
	poids = new double[nbpart];
	poidsPrec = new double[nbpart];
	
	particuleFinale = new int[nbvar];
	particuleFinalePrec = new int[nbvar];
	
	/* initialisation des matrices de variance *///ATTENTION : partie non générique
	switch (nbvar) {
		case nbvarEllipsoide:
			rwDyn = VARDynellipsoide;//en mm : en fait on aura une evolution comprise dans +-2.5*sqrt(valeur) (parcequ'on fait 3 sigma)
			Detect = VARDetectellipsoide;//le 0 est interdit!!!
			break;
		case nbvarSphere:
			rwDyn = VARDynsphere;//en mm : en fait on aura une evolution comprise dans +-2.5*sqrt(valeur) (parcequ'on fait 3 sigma)
			Detect = VARDetectsphere;//le 0 est interdit!!!
			break;
		default:
			cout << "particules.cpp : ERREUR, nb de variable non traite" << endl;
			exit(0);
	}
	/* inverses des matrices de covariance */
	invcovdetect = new double [nbvar];//inverse de la covariance selon la detection
	invcovdyn = new double [nbvar];//inverse de la covariance selon la dynamique
	for (i=0;i<nbvar;i++) {
		invcovdetect[i] = 1/(Detect[i]*Detect[i]);//var ne peut pas avoir de 0, et c'est une matrice diagonale, selon la detection
		invcovdyn[i] = 1/(rwDyn[i]*rwDyn[i]);//var ne peut pas avoir de 0, et c'est une matrice diagonale, selon la dynamique
	}
	
	/* Generateur aleatoire en fonction de l'horloge */
	alea = cvRNG((uint64)RDTSC());
	/* pour l'évolution de chaque particule : */
	aleaMat = cvCreateMat(nbvar*2*nbpart, 1, CV_32FC1);//on prend deux fois la taille de ce qu'on a besoin pour pouvoir faire de nouveaux tirs (quand on obtient pas ce qu'on veut)
	aleaVect = aleaMat->data.fl;//~=RandVectorMat : il contient toujours les donnees de RandVectorMat pour y acceder plus facilement
	/* pour le choix entre évolution suivant la dynamique ou suivant une détection : */
	alphaMat = cvCreateMat(nbpart, 1, CV_32FC1);//gestion de l'aleatoire
	alpha = alphaMat->data.fl;
	/* pour le choix de la particule détectée à utiliser pour la génération de particule : */
	bethaMat = cvCreateMat(nbpart, 1, CV_32FC1);//gestion de l'aleatoire
	betha = bethaMat->data.fl;

	diff = new int[nbvar];//différence de deux particules
	parttemp = new int[nbvar];//particule temporaire
	parttempdouble = new double[nbvar];//particule temporaire en double (pour la selection)
	
	/* génération des tirages aléatoires pour le premier tour après initialisation */
	cvRandArr(&alea, alphaMat, CV_RAND_UNI, cvScalar(0), cvScalar(1));//tirage uniforme entre 0 et 1
	cvRandArr(&alea, bethaMat, CV_RAND_UNI, cvScalar(0), cvScalar(1));//tirage uniforme entre 0 et 1
	cvRandArr(&alea, aleaMat, CV_RAND_NORMAL, cvRealScalar(0), cvRealScalar(1));//tirage aléatoire du bruit (distribution normale : m=0 sd=1)
	J=0;
	
	/* zones interdites */
	ZinterditesCentre = new int*[INTERDICTIONS_MAX];
	ZinterditesRayon = new int[INTERDICTIONS_MAX];
	nbZinterdites = 0;
	for (i=0;i<INTERDICTIONS_MAX;i++) {
		ZinterditesCentre[i] = new int[3];
	}
	
	/* somme pondérée des sinus et cosinus pour le calcul de la moyenne des angles *///...!!!ATTENTION : pas générique du tout, il faudrait un drapeau pour différencier les 2 types de variables
	S = new double[9];
	C = new double[9];
	
	selectionFaite = false;
}


/* destructeur :
 *
 */
Particules::~Particules() {

	for(register int i=0;i<nbpart;i++) {
		delete [] particules[i];
		delete [] particulesPrec[i];
		delete [] particuleDetectees[i];
	}

	delete [] particules;
	delete [] particulesPrec;
	delete [] particuleDetectees;

	delete [] poids;
	delete [] poidsPrec;

	delete [] particuleFinale;
	delete [] particuleFinalePrec;

	cvReleaseMat(&aleaMat);
	cvReleaseMat(&alphaMat);
	cvReleaseMat(&bethaMat);

	delete [] diff;
	delete [] parttemp;
	delete [] parttempdouble;

	delete [] invcovdetect;
	delete [] invcovdyn;

	/* zones interdites */
	for(register int i=0;i<INTERDICTIONS_MAX;i++) delete [] ZinterditesCentre[i];
	delete [] ZinterditesCentre;
	delete [] ZinterditesRayon;
	
	delete [] S;
	delete [] C;
}


/* affiche le vecteur d'état de la particule i */
void Particules::afficheParticule(int i) {
	
	cout << "particule [" << i << "] = ";
	for (j=0;j<nbvar;j++) cout << particules[i][j] << " ";
	cout << endl;
}


/* changer la matrice de variance de la dynamique */
bool Particules::changevar(int *var) {
	
	if (var==NULL) {
		cout << "particules.cpp (changevar) : ERREUR, matrice var vide" << endl;
		return false;
	}
	
	bool bien=true;
	
	for (int i=0; i<nbvar; i++) {
		if (var[i]!=0) {
			rwDyn[i] = var[i];//en mm : en fait on aura une évolution comprise dans +-2.5*valeur (parcequ'on fait 3 sigma)
			invcovdyn[i] = 1/(var[i]*var[i]);
		} else {
			cout << "particules.cpp (changevar) : ERREUR, aucune variance ne peut pas être nulle, pour celle-ci (" << i << ") on conserve donc le parametre precedemment etabli" << endl;
			bien = false;
		}
	}
	
	return bien;
}


/* genere une particule :
 * 
 * une certaine proportion des particules (prop) sont générées autour des détections, 
 * les autres sont générées autour des particules les mieux notées le tour précédent.
 * 
 * ATTENTION: la détection doit avoir eu lieu avant!
 * 
 * i : numéro de la particule à générer
 * 
 * la fonction retourne un coef lamda : coefficient multiplicateur pour l'évaluation de la particule (0 si la génération c'est mal passée)
 */
double Particules::genere(int i) {
	
	if (i==0) selectionFaite = false;//la génération de la première particule indique le début d'un nouveau tour
	
	ft=0;
	
	if (alpha[i] > prop) {//génération d'après la dynamique
		
		/* sélection de la particuleprécédente à partir de laquelle on génère cette nouvelle particule */
		cumul = 0;
		for (j=0;j<nbpartconservees;j++) { cumul+=poidsPrec[j]; if (cumul>betha[i]) break; }

		if (!evolutionDapresDynamique(i, j)) return 0;
		else return 1;//la dynamique n'influence pas l'évaluation de la particule (seulement si l'évolution c'est mal passée)
		
	} else {//génération d'après les détections
		
		if (nbpartdetectees==0) return 0;
		/* nouvelle particule tirée a partir de la particuleDetectee choisie au hasard */
		if ( evolutionDapresDetection(i, (int)((nbpartdetectees-1)*betha[i]) ) ) {
			somme = 0;
			for (j=0;j<nbvar;j++) somme += invcovdetect[j]*diff[j]*diff[j];
			
			/* calcul de gt : évaluation du tir (fonction d'importance) */ //gt = 1/nbparticulesdétectées * somme(0 à nbdétection)( exp( -1/2 * 1/nbvar * (particule - detection[i])t * invcov * (particule - detection[i]) ) )
			gt = invnbpartdetectees* exp( -0.5 * 1/nbvar * somme );//gain de temps : on approxime en ne faisant pas la somme sur toutes les detections : elles sont en general loin -> diff grand -> exp petit voir 0
			/* calcul de ft : évaluation de la dynamique */ //ft = somme(0 à nbparticulesprécédentesconservees)( poidsprécédent * exp( -1/2 * 1/nbvar * (particule - particuleprécédente[i])t * invcov * (particule - particuleprécédente[i]) ) )
			for(j=0;j<nbpartconservees;j++) {
				somme = 0;
				for (k=0;k<nbvar;k++) {
					diff[k] = particulesPrec[j][k] - particules[i][k];
					somme += invcovdyn[k]*diff[k]*diff[k];
				}

				ft+= poidsPrec[j] * exp( -0.5 * 1/nbvar * somme );
			}

			/* calcul de lamda */
			return ft/gt;
		}
		else return 0;//mauvaise detection
	}
}


/* selectionne la particule finale :
 * 
 * la particule finale est en fait la moyenne de toutes les particules pondérées par leur poids normalisé
 * 
 * retourne true en temps normal, false si tous les poids sont nuls (la particule finale n'est alors pas calculée)
 */
bool Particules::selectionne() {

	somme=0;
	
	/* on garde en mémoire la particule finale du tour précédent */
	if (!selectionFaite) {//on n'écrase particuleFinalePrec qu'une fois par tour (à la première sélection et non plus aux suivantes s'il y en a plusieurs)
		temp = particuleFinalePrec;
		particuleFinalePrec = particuleFinale;
		particuleFinale = temp;
		selectionFaite = true;
	}
	
	register int i;
	/* calcul de la somme des poids, sert pour la normalisation du poids des particules */
	for (i=0;i<nbpart;i++) somme+=poids[i];
	
	if (somme==0) return false;

	/* RAZ de la particule finale */
	memset(parttempdouble,0,nbvar*sizeof(double));
	memset(S,0,nbvar*sizeof(double));
	memset(C,0,nbvar*sizeof(double));

	register int l;
	/* calcul de la moyenne pondérée */
	for (i=0;i<nbpart;i++) {
		if(poids[i]>0) {
			poids[i]/=somme;//normalisation du poids
			switch (nbvar) {
				case nbvarEllipsoide:
					for (l=0;l<6;l++) parttempdouble[l] += particules[i][l]*poids[i];//ici, on a besoin d'additionner en double car la multiplication par le poids fait qu'on a souvent moins de 1 à rajouter
					/* pour les angles *///...!!!ATTENTION : pas générique du tout, il faudrait un drapeau pour différencier les 2 types de variables
					for (l=6;l<9;l++) {
						S[l] += sin(particules[i][l]*3.1415/180)*poids[i];
						C[l] += cos(particules[i][l]*3.1415/180)*poids[i];
					}
					break;
					
				case nbvarSphere:
					for (l=0;l<nbvar;l++) parttempdouble[l] += particules[i][l]*poids[i];//ici, on a besoin d'additionner en double car la multiplication par le poids fait qu'on a souvent moins de 1 à rajouter
					break;
					
				default:
					cout << "particules.cpp : ERREUR, nb de variable non traite" << endl;
					exit(0);
			}
		}
	}
	switch (nbvar) {
		case nbvarEllipsoide:
			for (l=0;l<6;l++) particuleFinale[l] = int(parttempdouble[l]);//ici, on a besoin d'additionner en double car la multiplication par le poids fait qu'on a souvent moins de 1 à rajouter
			/* pour les angles *///...!!!ATTENTION : pas générique du tout, il faudrait un drapeau pour différencier les 2 types de variables
			for (l=6;l<9;l++) particuleFinale[l] = int(atan(S[l]/C[l])*180/3.1415);
			break;
			
		case nbvarSphere:
			for (l=0;l<nbvar;l++) particuleFinale[l] = int(parttempdouble[l]);//ici, on a besoin d'additionner en double car la multiplication par le poids fait qu'on a souvent moins de 1 à rajouter
			break;
			
		default:
			cout << "particules.cpp : ERREUR, nb de variable non traite" << endl;
			exit(0);
	}
	
	return true;
}


/* reechantillonne les particules :
 *
 * les particules les moins probables disparaissent
 * les autres sont conservees ou demultipliees suivant leur poids d'après l'algorythme de Kitagawa
 * 
 * on conserve aussi en mémoire les particules assez vraissemblables pour former la base du prochain tour
 * il s'agit des nbpartconservees particules de particulesPrec (leurs poids respectifs étant stoqués dans poidsPrec)
 * elles peuvent servir de mémoire du tour précédent tant qu'on n'a pas rééchantillonné une nouvelle fois (mais poidsPrec est renormalisé)
 * 
 * ATTENTION : le poids doit avoir été normalisé avant!
 */
void Particules::reechantillonne() {

	modif=true;//pour ne pas recopier inutilement
	cumul = poids[0];//poids cumulé
	pas = 1/(double)nbpart;   
	nbpartMoinsUn = nbpart-1;

	/* sélection d'un point de départ aléatoire dans l'intervalle [0 pas] */
	Uj = pas*cvRandReal(&alea);//poids de comparaison
	nbpartconservees=0;
	
	register int i=0;
	/* redistribution */
	for(;Uj<1;Uj+=pas) {//redistribution de Kitagawa (le premier Uj est tiré aléatoirement, les suivant sont détermines a partir de lui), pour une redistribution classique on aurait fait un tirage aléatoire entre 0 et 1 a chaque boucle -> gain en temps de calcul
				//pour gagner encore du temps de calcul, j'ai modifié l'algo de Kitagawa pour ne plus recopier toutes les particules au fur et à mesure, mais l'esprit reste le même
				//au final on obtient uniquement les particules les plus importantes et leur poids
		while( (i<nbpartMoinsUn) && (Uj>cumul) ) { i++; cumul+=poids[i]; modif=true; }
		if (modif) {
			temp = particulesPrec[nbpartconservees];//pour ne pas avoir a recopier un vecteur entier, on échange simplement les adresses
			particulesPrec[nbpartconservees] = particules[i];
			particules[i] = temp;

			poidsPrec[nbpartconservees] = poids[i];//on conserve le poids de la particule conservée
			nbpartconservees++;
			modif=false;
		}
	}
	
	/* normalisation des poids conservés (nécessaire pour la génération des particules) *///les poids ne sont plus normalisés puisqu'on a éliminé des particules
	cumul = 0;
	for(i=0;i<nbpartconservees;i++) cumul+=poidsPrec[i];//le nombre de particules conservées étant en général faible, on gagne du temps en faisant comme çà plutôt qu'en préparant déjà les particules du tour prochain.
	for(i=0;i<nbpartconservees;i++) poidsPrec[i]/=cumul;

	/* génération des tirages aléatoires pour le tour prochain */
	cvRandArr(&alea, alphaMat, CV_RAND_UNI, cvScalar(0), cvScalar(1));//tirage uniforme entre 0 et 1
	cvRandArr(&alea, bethaMat, CV_RAND_UNI, cvScalar(0), cvScalar(1));//tirage uniforme entre 0 et 1
	cvRandArr(&alea, aleaMat, CV_RAND_NORMAL, cvRealScalar(0), cvRealScalar(1));//tirage aléatoire du bruit (distribution normale : m=0 sd=1)
	J=0;
	nbZinterdites = 0;
}


/* fonction équivalente de selectionne() + reechantillonne() pour une cible perdue :
 * 
 * retrouvee : false si la cible est perdue, true si la cible était perdue, mais a été retrouvée par reinitialisation douce
 *
 * 1) la particule finale précédente devient également la particule finale du nouveau tour
 * 2) on conserve en mémoire la particule finale comme étant l'unique particule conservée pour former la base du prochain tour
 * 3) réinitialisations necessaires avant chaque nouveau tour
 */
void Particules::selectionneEtReechantillonneApresPerte(bool retrouvee) {

	if (!retrouvee) {//si la particule a été retrouvée, ces deux étapes ont déjà eu lieu
		/* 1) la particule finale précédente devient également la particule finale du nouveau tour */
		if (selectionFaite) memcpy(particuleFinale, particuleFinalePrec, nbvar*sizeof(int));//si la sélection n'a pas encore eu lieu, particuleFinale contient la particuleFinale du pas précédent!
		
		/* 2) on conserve en mémoire la particule finale comme étant l'unique particule conservée pour former la base du prochain tour */
		nbpartconservees=1;
		poidsPrec[0]=1;
		memcpy(particulesPrec[0], particuleFinale, nbvar*sizeof(int));
	}
	
	/* 3) réinitialisations necessaires avant chaque nouveau tour */
	cvRandArr(&alea, alphaMat, CV_RAND_UNI, cvScalar(0), cvScalar(1));//tirage uniforme entre 0 et 1
	cvRandArr(&alea, bethaMat, CV_RAND_UNI, cvScalar(0), cvScalar(1));//tirage uniforme entre 0 et 1
	cvRandArr(&alea, aleaMat, CV_RAND_NORMAL, cvRealScalar(0), cvRealScalar(1));//tirage aléatoire du bruit (distribution normale : m=0 sd=1)
	J=0;
	nbZinterdites = 0;
}


/* réinitialise les zones interdites : (pour ne pas dépasser la limite inutilement)
 * 
 */
void Particules::RAZinterdictions() {

	nbZinterdites = 0;
}


/* génère une zone interdite sphérique :
 * 
 * centre : centre de la zone 3D à interdire
 * rayon : son rayon
 */
void Particules::interdire(int* centre, int rayon) {
	
	if (nbZinterdites==INTERDICTIONS_MAX) { cout << "particules.cpp : Trop d'interdictions!!!" << endl; return; }
	ZinterditesCentre[nbZinterdites][0] = centre[0];
	ZinterditesCentre[nbZinterdites][1] = centre[1];
	ZinterditesCentre[nbZinterdites][2] = centre[2];
	ZinterditesRayon[nbZinterdites] = rayon;
	nbZinterdites++;
}

/* génère une zone interdite 2D : la zone 2D est interdite quelque soit z
 * 
 * centre : centre de la zone à interdire
 * rayon : son rayon
 */
void Particules::interdire2D(int* centre, int rayon) {
	
	if (nbZinterdites==INTERDICTIONS_MAX) { cout << "particules.cpp : Trop d'interdictions!!!" << endl; return; }
	ZinterditesCentre[nbZinterdites][0] = centre[0];
	ZinterditesCentre[nbZinterdites][1] = centre[1];
	ZinterditesCentre[nbZinterdites][2] = 0;
	ZinterditesRayon[nbZinterdites] = rayon;
	nbZinterdites++;
}


/* traite les interdiction :
 * 
 * i : numéro de la particule dont on vérifie la validité
 * 
 * renvoie si la particule est dans une zone interdite où non.
 */
bool Particules::interdit(int i) {
	
	for (j=0;j<nbZinterdites;j++) {//pour toutes les zones interdites
		if (particules[i][0] < ZinterditesCentre[j][0]+ZinterditesRayon[j]) if (particules[i][0] > ZinterditesCentre[j][0]-ZinterditesRayon[j]) 
			if (particules[i][1] < ZinterditesCentre[j][1]+ZinterditesRayon[j]) if (particules[i][1] > ZinterditesCentre[j][1]-ZinterditesRayon[j]) {
				if (ZinterditesCentre[j][2]==0) return true;//pour les zones interdites 2D, z ne compte pas
				else {
					if (particules[i][2] < ZinterditesCentre[j][2]+ZinterditesRayon[j]) if (particules[i][2] > ZinterditesCentre[j][2]-ZinterditesRayon[j])
					return true;
				}
			}
	}
	
	return false;
}


/* traite les interdiction pour les détections :
 * 
 * i : numéro de la particule dont on vérifie la validité
 * 
 * renvoie si la particule est dans une zone interdite où non.
 */
bool Particules::detecteeinterdit(int i) {
	
	for (j=0;j<nbZinterdites;j++) {//pour toutes les zones interdites
		if (particuleDetectees[i][0] < ZinterditesCentre[j][0]+ZinterditesRayon[j]) if (particuleDetectees[i][0] > ZinterditesCentre[j][0]-ZinterditesRayon[j]) 
			if (particuleDetectees[i][1] < ZinterditesCentre[j][1]+ZinterditesRayon[j]) if (particuleDetectees[i][1] > ZinterditesCentre[j][1]-ZinterditesRayon[j]) {
				if (ZinterditesCentre[j][2]==0) return true;//pour les zones interdites 2D, z ne compte pas
				else {
					if (particuleDetectees[i][2] < ZinterditesCentre[j][2]+ZinterditesRayon[j]) if (particuleDetectees[i][2] > ZinterditesCentre[j][2]-ZinterditesRayon[j])
					return true;
				}
			}
	}
	
	return false;
}





/**********************/
/* fonctions internes */
/**********************/

/* génère une particule à partir de l'évolution dynamique d'une bonne particule du tour précédent :
 * 
 * i : numéro de la particule à générer
 * N : numéro de la particule précédente à partir de laquelle on génère cette particule
 * 
 * retourne si çà c'est bien passé ou non
 */
bool Particules::evolutionDapresDynamique(int i, int N) {//ATTENTION : fonction non générique

	limite = (i+1)*nbvar*2-1;//on a le droit à 2*nbvar numéros aléatoires par particule
	
	for (j=0;j<nbvar;j++) parttemp[j] = particulesPrec[N][j] + int (rwDyn[j]*aleaVect[J++]);

	/* sécurite : en cas d'incohérance complète */
	if (parttemp[2]<0) return false;//on ne peut pas avoir une distance négative (ou nulle)
	
	switch (nbvar) {
		case nbvarEllipsoide:
			/* régulation de la taille des demis-axes */
			/* 1) ils ne peuvent être < à tailleBlobMin ou > à tailleBlobMax */
			while ((parttemp[4]<tailleBlobMin)||(parttemp[4]>tailleBlobMax)) { if(J==limite)return false; parttemp[4] = particulesPrec[N][4] + int (rwDyn[4]*aleaVect[J++]); }
			while ((parttemp[5]<tailleBlobMin)||(parttemp[5]>tailleBlobMax)) { if(J==limite)return false; parttemp[5] = particulesPrec[N][5] + int (rwDyn[5]*aleaVect[J++]); }
			
			/* 2) le volume est considéré comme constant -> on s'en sert pour déterminer la profondeur */
			parttemp[3] = demivolumeMain / (parttemp[4] * parttemp[5]);//on reste en int, l'erreur sera assez faible pour être acceptable
			
			break;
			
		case nbvarSphere:
			//rien de particulier
			break;
			
		default:
			cout << "particules.cpp : ERREUR, nb de variable non traite" << endl;
			exit(0);
	}
	
	/* on ne prend aucune particule au delà d'une certaine distance */
	while (parttemp[2]>DISTMAXRECHERCHE) { if(J==limite)return false; parttemp[2] = particulesPrec[N][2] + int (rwDyn[2]*aleaVect[J++]); }
	while (parttemp[2]<DISTMINRECHERCHE) { if(J==limite)return false; parttemp[2] = particulesPrec[N][2] + int (rwDyn[2]*aleaVect[J++]); }
	
	/* la particule est acceptable -> on la met en place */
	temp = particules[i];//pour ne pas avoir a recopier un vecteur entier, on échange simplement les adresses
	particules[i] = parttemp;
	parttemp = temp;
	
	return true;
}


/* génère une particule à partir de l'évolution d'une détection :
 * 
 * i : numéro de la particule à générer
 * N : numéro de la détection à partir de laquelle on génère cette particule
 * 
 * retourne si çà c'est bien passé ou non
 */
bool Particules::evolutionDapresDetection(int i, int N) {//ATTENTION : fonction non générique
	
	limite = (i+1)*nbvar*2-1;//on a le droit à 2*nbvar numéros aléatoires par particule
	
	for (j=0;j<nbvar;j++) { diff[j]=int (Detect[j]*aleaVect[J++]); parttemp[j]=particuleDetectees[N][j] + diff[j]; }//on a besoin de la difference entre la detection et la particule obtenue

	switch (nbvar) {
		case nbvarEllipsoide:
			/* régulation de la taille des demis-axes */
			/* 1) ils ne peuvent être < à tailleBlobMin */
			while ((parttemp[4]<tailleBlobMin)||(parttemp[4]>tailleBlobMax)) { if(J==limite)return false; diff[4]=int (Detect[4]*aleaVect[J++]); parttemp[4] = particuleDetectees[N][4] + diff[4]; }
			while ((parttemp[5]<tailleBlobMin)||(parttemp[5]>tailleBlobMax)) { if(J==limite)return false; diff[5]=int (Detect[5]*aleaVect[J++]); parttemp[5] = particuleDetectees[N][5] + diff[5]; }
			
			/* 2) le volume est considéré comme constant -> on s'en sert pour déterminer la profondeur */
			parttemp[3] = demivolumeMain / (parttemp[4] * parttemp[5]);//on reste en int, l'erreur sera assez faible pour être acceptable
			break;
			
		case nbvarSphere:
			//rien de particulier
			break;
			
		default:
			cout << "particules.cpp : ERREUR, nb de variable non traite" << endl;
			exit(0);
	}

	/* la particule est acceptable -> on la met en place */
	temp = particules[i];//pour ne pas avoir a recopier un vecteur entier, on échange simplement les adresses
	particules[i] = parttemp;
	parttemp = temp;
	return true;
}

