/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef PARTICULES_H
#define PARTICULES_H
 
/** 
 * 
 * Cette classe contient les fonctions directement relatives aux particules : génération, sélection, etc.
 * Elle gère également les particules courantes et passées du filtre particulaire.
 * 
 * Les particules sont de dimension nbvar.
 * Certaines fonctions ne sont pas génériques, il faudra donc créer un nouveau cas dans celles-ci pour chaque dimension non encore prise en compte
 * Ces fonction sont : genere(), evolutionDapresDetection() et evolutionDapresDynamique()
 * 
 * \author BURGER Brice
 * \since 01/2007
 */


#include "gestParams.h"
#include "temps_cpu.h"
using namespace std;


class Particules
{

	public:
	
	/**************************************/
	/* variables publiques :              */
	/**************************************/
	
	/** nombre de particule */
	int nbvar;
	
	/** particules courantes du filtre particulaire */
	int** particules;
	/** particules du tour précédent *///en fait seules les particules au poids assez significatif sont conservées ici (i.e. conservées après rééchantillonnage)
	int** particulesPrec;
	int nbpartconservees;//nb de particules conservées après rééchantillonnage
	/** poids des particules courantes */
	double* poids;
	/** poids des particules du tour précédent */
	double* poidsPrec;
	/** particules détectées */
	int** particuleDetectees;
	int nbpartdetectees;//nb de particules détectées
	double invnbpartdetectees;// = 1/nbpartdetectees
	/** particule finale : particule issue de la traque à ce tour */
	int* particuleFinale;
	/** particule finale précédente : particule issue de la traque au tour précédent */
	int* particuleFinalePrec;
	/** si la sélection a été faite */
	bool selectionFaite;

	/* matrices de variance etc */
	int *rwDyn;
	int *Detect;
	double *invcovdetect;
	double *invcovdyn;
	

	
	/**************************************/
	/* fonctions publiques :              */
	/**************************************/
	
	/** constructeur :
	 * 
	 * \param Nbvar : nombre de variables des particules à créer
	 */
	Particules(int Nbvar);
	
	
	/** destructeur :
	 *
	 */
	~Particules();
	
	
	/** affiche le vecteur d'état de la particule i */
	void afficheParticule(int i);

	
	/** changer la matrice de variance de la dynamique */
	bool changevar(int *var);
	
	
	/** genere une particule :
	 * 
	 * une certaine proportion des particules (prop) sont générées autour des détections, 
	 * les autres sont générées autour des particules les mieux notées le tour précédent.
	 * 
	 * ATTENTION: la détection doit avoir eu lieu avant!
	 * 
	 * \param i : numéro de la particule à générer
	 * 
	 * \return la fonction retourne un coef lamda : coefficient multiplicateur pour l'évaluation de la particule (0 si la génération c'est mal passée)
	 */
	double genere(int i);
		
		
	/** selectionne la particule finale :
	 * 
	 * la particule finale est en fait la moyenne de toutes les particules pondérées par leur poids normalisé
	 * 
	 * \return retourne true en temps normal, false si tous les poids sont nuls (la particule finale n'est alors pas calculée)
	 */
	bool selectionne();
	
	
	/** reechantillonne les particules :
	 *
	 * les particules les moins probables disparaissent
	 * les autres sont conservees ou demultipliees suivant leur poids d'après l'algorythme de Kitagawa
	 * 
	 * on conserve aussi en mémoire les particules assez vraissemblables pour former la base du prochain tour
	 * il s'agit des nbpartconservees particules de particulesPrec (leurs poids respectifs étant stoqués dans poidsPrec)
	 * elles peuvent servir de mémoire du tour précédent tant qu'on n'a pas rééchantillonné une nouvelle fois
	 * 
	 * ATTENTION : le poids doit avoir été normalisé avant!
	 */
	void reechantillonne();
	
	
	/** fonction équivalente de selectionne() + reechantillonne() pour une cible perdue :
	 * 
	 * \param retrouvee : false si la cible est perdue, true si la cible était perdue, mais a été retrouvée par reinitialisation douce
	 *
	 * 1) la particule finale précédente devient également la particule finale du nouveau tour
	 * 2) on conserve en mémoire la particule finale comme étant l'unique particule conservée pour former la base du prochain tour
	 * 3) réinitialisations necessaires avant chaque nouveau tour
	 */
	void selectionneEtReechantillonneApresPerte(bool retrouvee);
	
	
	/** réinitialise les zones interdites : (pour ne pas dépasser la limite inutilement)
	 * 
	 */
	void RAZinterdictions();
	
	
	/** génère une zone interdite sphérique :
	 * 
	 * \param centre : centre de la zone 3D à interdire
	 * \param rayon : son rayon
	 */
	void interdire(int* centre, int rayon);
	
	
	/** génère une zone interdite 2D : la zone 2D est interdite quelque soit z
	 * 
	 * \param centre : centre de la zone à interdire
	 * \param rayon : son rayon
	 */
	void interdire2D(int* centre, int rayon);
	
	
	/** traite les interdiction :
	 * 
	 * \param i : numéro de la particule dont on vérifie la validité
	 * 
	 * \return renvoie si la particule est dans une zone interdite où non.
	 */
	bool interdit(int i);
	
	
	/** traite les interdiction pour les détections :
	 * 
	 * \param i : numéro de la particule dont on vérifie la validité
	 * 
	 * \return renvoie si la particule est dans une zone interdite où non.
	 */
	bool detecteeinterdit(int i);
	
	
	
	private:
	
	/**********************/
	/* variables internes */
	/**********************/
	
	/* générateur aléatoire en fonction de l'horloge */
	CvRNG alea;//générateur d'aléa
	/* pour l'évolution de chaque particule : */
	CvMat *aleaMat;//ce vecteur contiendra les valeurs aléatoires générées
	float* aleaVect;//pour accéder plus rapidement aux donnees de RandVectorMat
	/* pour le choix entre évolution suivant la dynamique ou suivant une détection : */
	CvMat* alphaMat;
	float* alpha;
	/* pour le choix de la particule détectée (ou conservée) à utiliser pour la génération de particule : */
	CvMat* bethaMat;
	float* betha;
	
	/* différence de deux particules */
	int* diff;
	/* particule temporaire */
	int* parttemp;
	double* parttempdouble;//sert uniquement à la sélection
	/* adresse temporaire */
	int* temp;
	/* zones interdites */
	int** ZinterditesCentre;
	int* ZinterditesRayon;
	int nbZinterdites;
	
	/* numéro du chiffre aléatoire en cours */
	int J;
	
	/* somme pondérée des sinus et cosinus pour le calcul de la moyenne des angles */
	double *S,*C;
	
	/* variables diverses */
	int j,k;//variables d'itération
	double gt, ft;//pour le calcul de alpha
	double cumul;//pour cumuler les poids
	double somme;//somme des poids
	bool modif;//pour ne pas recopier inutilement
	double Uj;//poids de comparaison
	double pas;   
	int nbpartMoinsUn;
	int limite;
	
	
	/**********************/
	/* fonctions internes */
	/**********************/
	
	/* génère une particule à partir de l'évolution dynamique d'une bonne particule du tour précédent :
	 * 
	 * i : numéro de la particule à générer
	 * N : numéro de la particule précédente à partir de laquelle on génère cette particule
	 * 
	 * retourne si çà c'est bien passé ou non
	 */
	bool evolutionDapresDynamique(int i, int N);
	
	
	/* génère une particule à partir de l'évolution d'une détection :
	 * 
	 * i : numéro de la particule à générer
	 * N : numéro de la détection à partir de laquelle on génère cette particule
	 * 
	 * retourne si çà c'est bien passé ou non
	 */
	bool evolutionDapresDetection(int i, int N);
	
};
#endif
