/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef IDMOT_H
#define IDMOT_H

/**
 * 
 * Ce fichier contient l'implémentation de l'algorithme IDMOT et de toutes les améliorations construites autour.
 * 
 * \author BURGER Brice
 * \since 04/2007
 */

#include <cv.h>
#include <highgui.h>
#include "gestParams.h"
#include "particules.h"
#include "evaluation.h"




/** Cette fonction initialise l'espace de travail :
 * 
 * \param nbcibles : nombre total de cible qu'il est possible de suivre
 */
void initIDMOT(int nbcibles);

/** Cette fonction relache l'espace de travail :
 * 
 * \param nbcibles : nombre total de cible qu'il est possible de suivre
 */
void finIDMOT(int nbcibles);

/** Cette fonction est un préambule à l'algorithme IDMOT : 
 * elle calcule les vecteurs vitesse des cibles suivies ainsi que leurs normes
 * elle doit être éxécutée en tout début d'un nouveau pas (avant que la particule finale du tour deux fois précédent n'ai disparue)
 * 
 * \param cibles : liste des poids non normalisés des particules de la première cible
 * \param nbcibles : tableau des particules générées ce tour pour la première cible
 * \param suivi : tableau de booléen indiquant pour chaque cible si elle est suivie ou non
 * 
 * \return elle renvoie les vitesses calculées (les normes via Vnorme et les vecteurs via V)
 */
void preambuleIDMOT(Particules** cibles, int nbcibles, bool* suivi);

/** Cette fonction est l'implémentation de l'algorithme IDMOT : 
 * elle influence le poids des particules afin d'empécher la fusion ou l'échange de ces cibles
 * elle doit être éxécutée après un nouveau pas, après la sélection, mais avant le rééchantillonnage
 * 
 * \param cibles : liste des poids non normalisés des particules de la première cible
 * \param nbcibles : tableau des particules générées ce tour pour la première cible
 * \param suivi : tableau de booléen indiquant pour chaque cible si elle est suivie ou non
 * 
 * suivi est également mis à jour en fonction des modifications apportées par l'algorithme
 */
void IDMOT(Particules** cibles, int nbcibles, bool* suivi, double** g, double* gmax);

/**********************/
/* fonctions internes */
/**********************/

/** Ceci est une sous-fonction du préambule à l'algorithme IDMOT : 
 * elle calcule le vecteur vitesse d'une cible ainsi que sa norme
 * 
 * \param particuleFinale : tableau contenant la particule finale (en réalité celle du tour précédent, puisqu'on est au tout début du pas)
 * \param particuleFinalePrec : tableau contenant la particule finale précédente (en réalité celle du tour deux fois précédent, puisqu'on est au tout début du pas)
 * 
 * \return on renvoie la vitesse calculée (la norme directement, le vecteur via V)
 */
int calculVitesse(int* particuleFinale, int* particuleFinalePrec, int* V);

/** Cette fonction est l'implémentation de l'algorithme IDMOT pour deux cibles : 
 * elle influence le poids des particules afin d'empécher la fusion ou l'échange de ces cibles
 * c'est une sous-fonction de IDMOT()
 * 
 * \param cibleI : première cible à traiter
 * \param gI : liste des vraissemblances non normalisés des particules de la première cible
 * \param gmaxI : maximum des gI (son adresse en fait, pour pouvoir le modifier)
 * \param VI : vecteur vitesse voulue pour la cible I (vitesse du tour précédent)
 * \param VnormeI : norme de VI
 * 
 * \param cibleJ : seconde cible à traiter
 * \param gJ : liste des vraissemblances non normalisés des particules de la seconde cible
 * \param gmaxJ : maximum des gJ (son adresse en fait, pour pouvoir le modifier)
 * \param VJ : vecteur vitesse voulue pour la cible J (vitesse du tour précédent)
 * \param VnormeJ : norme de VJ
 * 
 * \return on renvoie CI et CJ qui indiquent si les cibles sont encore suivies (des addresses doivent être données en entrée)
 */
void IDMOTdeuxcibles(Particules* cibleI, double* gI, double *gmaxI, bool* CI, int VnormeI, int* VI, 
			Particules* cibleJ, double* gJ, double *gmaxJ, bool* CJ, int VnormeJ, int* VJ);

/** Cette fonction applique une correction calculée à une particule et à sa cible : 
 * elle modifie dans un premier temps la vraissemblance de la particule
 * elle modifie ensuite le poids des particules et fait une sélection
 * 
 * \param cible : cible à traiter
 * \param g : vraissemblances des particules de la cible
 * \param i : numéro de la particule à corriger
 * \param gmax : maximum des vraissemblances des particules de la cible (l'adresse de la variable, en fait, pour pouvoir la modifier)
 * \param phi1 : correction par la "fonction de répulsion magnétique"
 * \param phi2 : correction par la "fonction d'inertie"
 * 
 * \return retourne si la cible reste suivie ou non (résultat de la sélection)
 */
bool correction(Particules* cible, double* g, int i, double *gmax, float phi1, float phi2);

/** phi1 : "fonction de répulsion magnétique" : 
 * elle est fonction de la distance entre la iième particule de la cible I et la particule finale de la cible J
 * 
 * \param particuleIi : iième particule de la cible I
 * \param particuleFinaleJ : particule finale de la cible J
 * \param bethahistI : fonction de déséquilibre historique (rajout par rapport à IDMOT de base)
 * 
 * \return retourne la correction a appliquer
 */
float Phi1(int *particuleIi, int *particuleFinaleJ, float bethahistI);

/** phi2 : "fonction d'inertie" : 
 * elle est fonction de la distance entre de la continuité en vitesse et en angle de la particule à traiter
 * 
 * \param particulei : iième particule de la cible à traiter
 * \param particuleFinalePrec : particule finale de la cible à traiter
 * \param bethahist : fonction de déséquilibre historique (rajout par rapport à IDMOT de base)
 * \param Vnorme : norme du vecteur vitesse du tour précédent
 * \param V : vecteur vitesse du tour précédent
 * 
 * \return retourne la correction a appliquer
 */
float Phi2(int *particulei, int *particuleFinalePrec, float bethahist, int Vnorme, int* V);


#endif
