/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef TRAQUEURDEDEUXMAINS_H
#define TRAQUEURDEDEUXMAINS_H

/** 
 * 
 * Programme principal, il fait l'interface entre le main ou genom et le reste du code.
 * Ici sont implémentés les fonctions de haut niveau du traqueur 3D à filtre particulaire.
 * Le but de ce programme est de traquer les deux mains d'un bonhomme dans l'espace (forme ellipsoidale) à partir des images d'un banc stéréo.
 * 
 * 
 * CODE COULEUR DE L'AFFICHAGE : (RGB)
 * 
 * \li détections -> rouge (100)
 * \li main gauche -> jaune (110) à vert (010)
 * \li main droite -> violet (101) à bleu (001)
 * \li tête -> noir (000) à blanc (111)
 * 
 * 
 * \author BURGER Brice
 * \since 02/2007
 */


#include "particules.h"
#include "evaluation.h"
#include "projection.h"
#include "detectionBlobPeau.h"
#include "detectionVisage.h"
#include "mesureMvt3D.h"
#include "gestParams.h"
#include "IDMOT.h"
#include "mesureDistance3D.h"
#include <sys/time.h>



class Traqueur2
{

	public:
	
	/**************************************/
	/* fonctions générales :              */
	/**************************************/
	
	/** constructeur du traqueur 3D :
	 * 
	 * \param imgG et imgD : images représentatives de celles qui serviront ensuite durant la traque (même taille, même convention de couleur, ...)
	 */
	Traqueur2(IplImage* imgG, IplImage* imgD);


	/** destructeur du traqueur 3D :
	 *
	 */
	~Traqueur2();
	
	
	/** changement d'espace de couleur
	 * 
	 * \return retourne true si tout c'est bien passé, false autrement (mauvais espace de couleur)
	 */
	bool ChangeEspaceCouleur(int EspaceCoul);
	
	
	/** fonction d'initialisation globale du traqueur : (il peut démarrer avec la tête seule)
	 * 
	 * \param imgG,imgD : images pour l'affichage (gauche et droite)
	 * \param imgTG,imgTD : images de travail (gauche et droite)
	 * \param - les images d'affichage et de travail peuvent être les mêmes
	 * 
	 * Ici on initialise à partir d'une détection de tête
	 * ATTENTION : il ne doit y avoir qu'un seul visage !! Sinon c'est le premier qui est pris.
	 * 
	 * \return retourne l'état du système (via l'énumération ETAT_GEST) :
	 * \return 	RIEN=0,			//rien n'est traqué
	 * \return 	TOUT=1,			//tout est traqué
	 * \return 	PASmAINgAUCHE=2,	//tout est traqué sauf la main gauche
	 * \return 	PASmAINdROITE=3,	//tout est traqué sauf la main droite
	 * \return 	AUCUNEmAIN=4,		//seule la tête est traquée
	 * \return 	PAStETE=5,		//tout est traqué sauf la tête
	 * \return 	PAStETEnImAINgAUCHE=6,	//seule la main droite est traquée
	 * \return 	PAStETEnImAINdROITE=7	//seule la main gauche est traquée
	 */
	ETAT_GEST INIT(IplImage* imgG,IplImage* imgD , IplImage* imgTG,IplImage* imgTD);
	
	
	/** pour avancer d'un pas dans le filtre particulaire :
	 * 
	 * \param imgG,imgD : images pour l'affichage (gauche et droite)
	 * \param imgTG,imgTD : images de travail (gauche et droite)
	 * \param - les images d'affichage et de travail peuvent être les mêmes
	 * 
	 * \return retourne l'état du système (via l'énumération ETAT_GEST) :
	 * \return 	RIEN=0,			//rien n'est traqué
	 * \return 	TOUT=1,			//tout est traqué
	 * \return 	PASmAINgAUCHE=2,	//tout est traqué sauf la main gauche
	 * \return 	PASmAINdROITE=3,	//tout est traqué sauf la main droite
	 * \return 	AUCUNEmAIN=4,		//seule la tête est traquée
	 * \return 	PAStETE=5,		//tout est traqué sauf la tête
	 * \return 	PAStETEnImAINgAUCHE=6,	//seule la main droite est traquée
	 * \return 	PAStETEnImAINdROITE=7	//seule la main gauche est traquée
	 */
	ETAT_GEST PAS(IplImage* imgG, IplImage* imgD , IplImage* imgTG,IplImage* imgTD);
	
	
	
	/*****************************************/
	/* fonctions de récupération :           */
	/*****************************************/
	
	/** retourne le vecteur d'état global : (TETE,MG,MD)
	 * 
	 * si l'une des cible n'est plus traquée, c'est son dernier vecteur d'état correct qui est renvoyé
	 */
	int* RECUP();
	
	/** retourne la taille du vecteur d'état : */
	int GETNBVAR();
	
	/** si la tête a été détectée ou non ce tour (par la détection de visage != traquée) : */
	bool TETEDETECTEE();
	
	
	
	/*****************************************/
	/* fonctions de modification d'obtions : */
	/*****************************************/
	
	/**
	 * fonctions pour changer la matrice de variance de la dynamique
	 * 
	 * \param typeParticule : type de particule qu'on veut affecter par cette modification
	 * \param var : nouvelle matrice de variance (ATTENTION : la taille de la matrice donnée doit correspondre à celle modifiée)
	 * 
	 * \return retourne true s'il n'y a pas eu de problème, false autrement
	 */
	bool changevar(TYPE_PARTICULE_GEST typeParticule, int *var);
	
	/** si l'on veut utiliser une interface graphique :
	 * 
	 * \param oui : true si on veux activer l'interface, false pour la désactiver
	 */
	void avec_interface(bool oui);
	
	/** pour modifier la position de la fenètre :
	 * 
	 * \param w,h : position de la fenètre
	 */
	void position_interface(int w, int h);
	
	/** si l'on veut afficher une image supplémentaire : proba peau, mouvement, distance...
	 * 
	 * \param oui : true si on veux afficher une image supplémentaire, false pour désactiver cet affichage
	 * \param gauche : true si l'image supplémentaire doit être basée sur l'image gauche, false pour la droite
	 * 
	 * Pour afficher cette image supplémentaire, il faut que l'affichage fonctionne déjà!
	 */
	void affiche_image_sup(bool oui, bool gauche=true);

	/** si l'on veut afficher l'image de proba peau :
	 * 
	 * Pour afficher cette image, il faut que l'affichage d'une image supplémentaire fonctionne déjà!
	 * 
	 * \param oui : true si on veux afficher cette image, false pour désactiver cette option
	 */
	void affiche_proba_peau(bool oui);

	/** si l'on veut afficher l'image de mouvement :
	 * 
	 * Pour afficher cette image, il faut que l'affichage d'une image supplémentaire fonctionne déjà!
	 * 
	 * \param oui : true si on veux afficher cette image, false pour désactiver cette option
	 */
	void affiche_mvt(bool oui);

	/** si l'on veut afficher l'image de distance :
	 * 
	 * Pour afficher cette image, il faut que l'affichage d'une image supplémentaire fonctionne déjà!
	 * 
	 * \param oui : true si on veux afficher cette image, false pour désactiver cette option
	 */
	void affiche_dist(bool oui);

	/** si l'on veut afficher les particules :
	 * 
	 * Pour afficher les particules, il faut que l'affichage de l'image de proba peau fonctionne déjà!
	 * 
	 * \param oui : true si on veux afficher les particules, false pour désactiver cette option
	 */
	void affiche_particules(bool oui);

	/** si l'on veut sauvegarder les images et/ou les résultats obtenues :
	 * 
	 * \param ouiImages : true pour sauvegarder les images, false pour désactiver cette option
	 * \param ouiResultats : true pour sauvegarder les résultats, false pour désactiver cette option
	 * \param cheminSauvegarde : chemin du dossier dans lequel les images seront sauvegardées (NULL pour avoir le chemin par défaut)
	 * 
	 * \return renvoie true en temps normal, false s'il y a un problème avec le fichier (mauvais chemin la plupart du temps)
	 */
	bool sauvegarde(bool ouiImages, bool ouiResultats, char* cheminSauvegarde);
	
	
	
	/*****************************************/
	/* fonctions d'affichage :               */
	/*****************************************/
	
	/** enregistre les images données en entrée séparément (et les affiches éventuellement) */
	void enregistreImgs (IplImage* imgg, IplImage* imgd);

	/** affichage sur l'ecran du robot : 2 images l'une en dessous de l'autre */
	void affiche (IplImage* imgg, IplImage* imgd);

	/** affichage sur l'ecran du robot : 3 images l'une en dessous de l'autre */
	void affiche (IplImage* imgg, IplImage* imgd, IplImage* img3);
	
	
	
	private :
	
	/**********************/
	/* fonctions internes */
	/**********************/
	
	/** calcule de la zone maximum (i.e. d'initialisation) de recherche d'une cible :
	 * les zones de recherche sont stoquées sous forme de leurs centres et taille de demi-axes (contenus respectivement dans centres et zones3D)
	 * 
	 * \param type : cible pour laquelle calculer la zone
	 * 
	 * \return retourne true si tout s'est bien passée, false autrement
	 */
	bool calculeZoneMax(TYPE_PARTICULE_GEST type);
	
	/** fonction de réinitialisation du traqueur : 
	 * 
	 * \param type : type de particule à réinitialiser
	 * 
	 * Ici on essaie de sauver un pas défectueux pour éviter une réinitialisation globale (via INIT) faisant perdre la continuité de la traque.
	 * Aucune nouvelle détection n'est faite ici.
	 * 
	 * \return retourne true si la réinitialisation s'est bien passée, false autrement
	 * \return on réactualise également le niveau de réinitialisation
	 */
	bool reinit(TYPE_PARTICULE_GEST type);
	
	/** fonction évaluant une particule via les fonctions d'évaluation 2D :
	 * 
	 * \param particule : la particule à évaluer
	 * \param imgG, imgD : images en cours
	 * \param type : type de particule à réinitialiser (et donc type d'évaluation à utiliser)
	 * 
	 * \return renvoie son évaluation (entre 0 et 1)
	 */
	double eval(int *particule, IplImage* imgG, IplImage* imgD, TYPE_PARTICULE_GEST type);
	
	/** fonction écrivant les résultats du traqueur dans le fichier de résultats courant :
	 * 
	 */
	void ecritResultats();
	
	/** fonction effectuant les vérifications qu'opencv ne gère pas sur les rectangles :
	 * pour définir une ROI comme pour dessiner un rectangle sur une image, il faut :  rect.w et rect.h >= 5  ,  0 <= rect.x et rect.y < w et h
	 * 
	 * \param rect : le rectangle à vérifier
	 */
	void verifRectOpencv(CvRect& rect);
	
	/**************************************/
	/* variables internes :               */
	/**************************************/
	
	/* état du système */
	ETAT_GEST ETAT;//état global
	bool *suivi;//tableau contenant l'état de chaque cible
	bool *T, *MG, *MD;//état de chaque cible (sous forme d'adresse : font partie de suivi)
	int* VEGlobal;//vecteur d'état global
	int nbvar;//nombre de variables du vecteur d'état
	
	/* images temporaires pour l'affichage */
	IplImage* imgSup;
	IplImage* img;
	IplImage *imageNG[3];
	
	/* adresses temporaires */
	int* temp, *temp2;
	CvRect* rect;//rectangle temporaire
	
	/* pour sauvegarder les images et les résultats */
	char* CheminSauvegarde;
	char* nomsauvegarde;
	FILE* fichierResultats;
	int numero;//pour sauvegarder des images
	
	/* particules */
	int nbcibles;//nombre de cibles à suivre
	Particules **cibles;//tableau de Particules contenant l'ensemble des cibles
	Particules *particulesMG;
	Particules *particulesMD;
	Particules *particulesTETE;
	
	Particules *particuleTemp;
	
	/* détecteurs */
	detectionBlobPeau *detecteurBlob;
	detectionBlobPeau *detecteurBlobTETE;
	detectionVisage *detecteurVisage;
	
	/* mesures */
	MesureMvt3D *mesureMvt;
	MesureDistance3D* mesureDistance;
	
	/* outils */
	projection *proj;
	evaluation *eval2D;
	
	/* drapeau d'options : */
	bool AVEC_INTERFACE;		//avec ou sans interface graphique
	bool AFFICHE_SUP;		//avec ou sans une image suplémentaire (une des suivantes)
	bool AFFICHE_SUP_GAUCHE;	//affichage suplémentaire basé sur l'image gauche (droite si false)
	bool AFFICHE_PROBA_PEAU;	//affiche l'image de proba peau
	bool AFFICHE_MVT;		//affiche l'image de mouvements
	bool AFFICHE_DIST;		//affiche l'image de distance
	bool AFFICHE_PARTICULES;	//si on affiche les particules ou non
	bool SAUVEGARDE_IMAGES;		//si on sauvegarde les images ou non
	bool SAUVEGARDE_RESULTATS;	//si on sauvegarde les résultats ou non
	
	/* pour la projection */
	int* projections;
	CvPoint point2dgauche, point2ddroite;
	CvSize axesg, axesd;
	int angleg, angled;
	
	/* pour l'évaluation des particules */
	double **g;//tableau contenant l'ensemble des vraissemblances de toutes les cibles (contient tous les g)
	double *gMG, *gMD, *gTETE;//tableau des vraissemblances de chaque cible
	double *gmax;//tableau contenant tous les gmax
	double *gmaxMG, *gmaxMD, *gmaxTETE;//maximums des g (sous forme d'adresse : font partie de gmax)
	double lamdaMG, lamdaMD, lamdaTETE;
	double* evalFinale;//résultats de l'évaluation de la particule finale
	
	/* pour la vérification de la distance Tête/mains */
	int X,Y,Z;
	
	/* zones de recherche */
	CvRect** zones;//zone de recherche pour chaque cible sur l'image gauche et droite
	int** zones3D;//mêmes zones, mais en 3D (contient les demis axes du paralélépipède)
	int** centres;//centre des zones de recherche (utilisés dans le cas des zones maximales)
	int rayon,L,H;//largeur et hauteur de la zone 2D (rayon pour les zones cubiques)
	int* niveau_reinit;//niveau de la reinitialisation pour chaque cible de 0 (pas de reinitialisation) à NIVEAU_REINIT_MAX (zone maximum = celle d'initialisation)

};
#endif
